; File: LEDandPb.asm
; Date: June 2, 2006
; Author: Julio Sanchez
; Processor: 16F84A
;
; Description:
; Circuit with LED wired to RB0 and pushbutton swtich,
; active low, wired to RA0. Pushbutton action turns LED
; OFF when pressed and ON when released.
;===========================
;        switches
;===========================
; Switches used in __config directive:
;   _CP_ON      	Code protection ON/OFF     
; * _CP_OFF      
; * _PWRTE_ON     Power-up timer ON/OFF
;   _PWRTE_OFF     
;   _WDT_ON       Watchdog timer ON/OFF  
; * _WDT_OFF       
;   _LP_OSC       Low power crystal occilator
; * _XT_OSC       External parallel resonator/crystal ocillator  
;   _HS_OSC       High speed crystal resonator (8 to 10 MHz)
;                 Resonator: Murate Erie CSA8.00MG = 8 MHz   
;   _RC_OSC       Resistor/capacitor ocillator (simplest, 20% error)
; |
; |_____ * indicates setup values

	processor 16f84A
	include	  <p16f84A.inc>
	__config  _XT_OSC & _WDT_OFF & _PWRTE_ON & _CP_OFF
;=====================================================
;              variables in PIC RAM
;=====================================================
; Declare variables at 2 memory locations
j	equ		0x0c
k	equ		0x0d
;========================================================
;                 m a i n   p r o g r a m
;========================================================
	org	0	  ; start at address 0
	goto	main
;=============================
; space for interrupt handler
;=============================
	org		0x04
;=============================
;       main program
;=============================
main:
; Initialize all lines in port B for output
	movlw	B'00000000'	; w = 00000000 binary
	tris	PORTB		; Set up port B for output
; Initialize port A, line 0, for input
	movlw	B'00000001'	; w = 00000001 binary
	tris	PORTA		; Set up RA0 for input
; Program loop to test state of pushbutton switch
;==============================
;   read PB switch state
;==============================
LEDctrl:
; Push button switch on demo board is wired to port A bit 0
; Switch logic is active low
	btfss	PORTA,0		; Test and skip if bit is set
	goto	turnOFF		; Turn LED off routine
; At this point port A bit 0 is set (switch is off)
; Turn ON line 0 in port B.
	bsf		PORTB,0		; RB0 high
	goto	LEDctrl
turnOFF:
; Routine to turn OFF LED
	bcf		PORTB,0		; RB0 low
	goto	LEDctrl

		end	

