; File name: RamDemo.asm
; Last revision: May 27, 2006
; Author: Julio Sanchez
; PIC: 16F877
;
; Description:
; Program to demonstrate access to General Purpose Register
; located in different banks. 16F877 GPR space is as
; follows:
; BANK 0       BANK 1        BANK 2       BANK 3
;  0x20        0xa0          0x110        0x190 |
;  --          --            --           --    | bank
;  --          --            --           --    | related
;  --          0xef          0x16f        0x1ef |
;  --          0xf0          0x170        0x1f0 |
;  0x70      <==  access registers at 0x70-0x7f
;  0x7f           these GPRs are common to all banks
;
; Program is desgined to be used with a debugger so that
; access to different registers can be tested
; GPR registers are named according to the following format:
;
;             REGx_yyy
;                | |||
;                | |||_________ hex address (up to 3 digits)
;                |_____________ bank number (0 to 3)
;
; CONCLUSIONS:
; GPRs located between 0x70 and 0x7f can be accessed from
; any bank. GPRs at other addresses require previous bank
; selection
;
;===========================
;      16F877 switches
;===========================
; Switches used in __config directive:
;   _CP_ON     	  Code protection ON/OFF     
; * _CP_OFF      
; * _PWRTE_ON     Power-up timer ON/OFF
;   _PWRTE_OFF  
;   _BODEN_ON     Brown-out reset enable ON/OFF
; * _BODEN_OFF 
; * _PWRTE_ON     Power-up timer enable ON/OFF
;   _PWRTE_OFF   
;   _WDT_ON       Watchdog timer ON/OFF  
; * _WDT_OFF
;   _LPV_ON       Low voltage IC programming enable ON/OFF
; * _LPV_OFF
;   _CPD_ON       Data EE memory code protection ON/OFF
; * _CPD_OFF
; OSCILLATOR CONFIGURATIONS:        
;   _LP_OSC       Low power crystal occilator
;   _XT_OSC       External parallel resonator/crystal ocillator  
; * _HS_OSC       High speed crystal resonator
;   _RC_OSC       Resistor/capacitor ocillator
; |               (simplest, 20% error)
; |
; |_____ * indicates setup values presently selected

	processor 	16f877		; Define processor
	#include	<p16f877.inc>
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _HS_OSC & _WDT_OFF & _LVP_OFF & _CPD_OFF
; __CONFIG directive is used to embed configuration data
; within the source file. The labels following the directive
; are located in the corresponding .inc file.
	errorlevel -302
; Supress bank-related warning 
;============================================================
;                          M A C R O S
;============================================================
; Macros to select the register banks
Bank0	MACRO			; Select RAM bank 0
		bcf	STATUS,RP0
		bcf	STATUS,RP1
		ENDM

Bank1	MACRO			; Select RAM bank 1
		bsf	STATUS,RP0
		bcf	STATUS,RP1
		ENDM

Bank2	MACRO			; Select RAM bank 2
		bcf	STATUS,RP0
		bsf	STATUS,RP1
		ENDM

Bank3	MACRO			; Select RAM bank 3
		bsf	STATUS,RP0
		bsf	STATUS,RP1
		ENDM
;=====================================================
;                  PIC register equates
;=====================================================
reg0_20	equ		0x20
reg0_50	equ		0x50
reg0_7e	equ		0x7e
reg1_a0	equ		0xa0
;============================================================
;                        P R O G R A M
;============================================================
		org		0	  ; start at address 
		goto	main
; Space for interrupt handlers
	org		0x08
main:
	Bank0
	nop
	movlw	0xee		; Test value
	movwf	reg0_20
	movwf	reg0_7e
;
	Bank1
; Register at address 0x20 in bank 0 cannot be accessed
	nop
	movf	reg0_20,w
	nop
; However, the register at 0x7e CAN be accessed
	movf	reg0_7e,w 
	nop
	Bank0
	movlw	0x0
	movf	reg0_7e,w
	nop
; How about from bank 3
	Bank3
	movlw	0x0
	movf	reg0_7e,w
	nop

loopHere:
	goto	loopHere
;============================================================
	end         ; END OF PROGRAM 
;============================================================     

