; File: Watchdog.asm
; Date: May 2, 2006
; Author: Julio Sanchez
;
; Description:
; Program to demonstrate the use of the watchdog timer
; in breaking out of an endless loop.
; A LED on port B, line 1, flashes on and off at 1/2
; second intervals for 20 iterations. At that time the
; program enters and endless loop. The watchdog timer
; times-out and restart the program
;===========================
;        switches
;===========================
; Switches used in __config directive:
;   _CP_ON      	Code protection ON/OFF     
; * _CP_OFF      
; * _PWRTE_ON     Power-up timer ON/OFF
;   _PWRTE_OFF     
; * _WDT_ON       Watchdog timer ON/OFF  
;   _WDT_OFF       
;   _LP_OSC       Low power crystal occilator
; * _XT_OSC       External parallel resonator/crystal ocillator  
;   _HS_OSC       High speed crystal resonator (8 to 10 MHz)
;                 Resonator: Murate Erie CSA8.00MG = 8 MHz   
;   _RC_OSC       Resistor/capacitor ocillator (simplest, 20% error)
; |
; |_____ * indicates setup values

;=========================
; setup and configuration
;=========================
	processor 16f84A
	include	  <p16f84A.inc>
	__config  _XT_OSC & _WDT_ON & _PWRTE_ON & _CP_OFF

;=====================================================
;                  PIC register equates
;=====================================================
porta	equ		0x05
portb   equ     0x06
status  equ		0x03
z		equ		0x02
tmr0	equ		0x01
;
;=====================================================
;              variables in PIC RAM
;=====================================================
; Local variables
	cblock	0x0d		; Start of block
	J			; counter J
	K			; counter K
	count1		; Auxiliary counter
	count2		; Second auxiliary counter
	old_w		; Context saving 
	old_status  ; Idem
	endc

;========================================================
;                 m a i n   p r o g r a m
;========================================================
	org	0	  ; start at address 0
	goto	main
;
;=============================
;      interrupt handler
;=============================
	org		0x04
;=============================
;       main program
;=============================
main:
; Setting the prescaler to the watchdog timer following
; the sequence reccomended by Microchip
	movlw	b'10010101'	; Clock source and some prescaler
	option
	clrf	tmr0		; Clear timer and prescaler
	movlw	b'10111101' ; WDT, do not change prescale 
	option				; again
; Reset watchdog timer
	clrwdt
; Final setting of OPTION register
	movlw	b'10111000'
; Set up the OPTION regiser bit map
;   7  6  5  4  3  2  1  0 <= OPTION bits
;   |  |  |  |  |  |__|__|_____ PS2-PS0 (prescaler bits)
;   |  |  |  |  |               Values for WDT
;   |  |  |  |  |               000 = 1:1   001 = 1:2
;   |  |  |  |  |               010 = 1:4   011 = 1:8
;   |  |  |  |  |               100 = 1:16  101 = 1:32
;   |  |  |  |  |               110 = 1:64 *111 = 1:128
;   |  |  |  |  |______________ PSA (prescaler assign)
;   |  |  |  |                 *1 = to WDT
;   |  |  |  |                  0 = to Timer0
;   |  |  |  |_________________ TOSE (Timer0 edge select)
;   |  |  |                     0 = increment on low-to-high
;   |  |  |                    *1 = increment in high-to-low
;   |  |  |____________________ TOCS (TMR0 clock source)
;   |  |                        0 = internal clock
;   |  |                       *1 = RA4/TOCKI bit source
;   |  |_______________________ INTEDG (Edge select)
;   |                          *0 = falling edge
;   |__________________________ RBPU (Pullup enable)
;                               0 = enabled
;                              *1 = dissabled
	option
	movlw	b'00000000' ; Port B is output
	tris	portb		; all others are output
	clrf	portb		; All port B to 0
; Port A is not used by this program
;============================
;    flash LED 20 times
;============================
; Program flashes LED wired to port B, line 2
; 5 times before entering the endless loop
	movlw	D'5'		; Number of iterations
	movwf	count2		; To counter
lights:
	movlw	b'00000010'	; Mask with bit 1 set
	xorwf	portb,f		; Complement bit 1 	
	call	long_delay
	call	long_delay
	call	long_delay	
	decfsz	count2,f		; Decrement counter
	goto	lights
	clrwdt					; Clear watchdog
;============================
;        endless loop
;============================
endless:
	goto	endless

;=============================
;     delay sub-routine
;=============================
long_delay
		movlw	D'200'	; w = 20 decimal
		movwf	J		; J = w
jloop:	movwf	K		; K = w
kloop:	decfsz	K,f		; K = K-1, skip next if zero
		clrwdt
		goto 	kloop
		decfsz	J,f		; J = J-1, skip next if zero
		clrwdt
		goto	jloop
		return

		end	
